/*
** Module   : BrowseColumnConfig.java
** Abstract : BrowseColumn widget container
**
** Copyright (c) 2005-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 SIY 20050912   @22822 Created initial version.
** 002 SIY 20051129   @23554 Changed PRIVATE-DATA attribute type to 
**                           String.
** 003 SIY 20060209   @24489 Major refactoring.
** 004 SIY 20060315   @25121 Changes required for complete BROWSE 
**                           implementation.
** 005 SIY 20060328   @25272 Added missing getFormat() method. 
** 006 GES 20060803   @28391 Made externalizable for better performance.
**                           Many code standard cleanups.
** 007 GES 20061025   @30638 Added transient content alignment flag 
**                           required by column data type override 
**                           (@ fields). Note that this flag is not 
**                           transferred nor copied.
** 008 ECF 20070928   @35273 Removed nullable field and related methods.
**                           Nullable constraints are checked at the
**                           server when a row is updated.
** 009 SIY 20080603   @38473 Added COLUMN-LABEL attribute.
** 010 SIY 20080703   @39037 Added set/getD/PfColor() methods as aliases 
**                           for appropriate set/getColumnD/PfColor().
** 011 SIY 20090122   @41205 Now setLabel() also sets column label.
** 012 SIY 20090206   @41261 Rolled back #011 due to regressions.
** 013 SIY 20090218   @41327 Added support for AT base field format width.
** 014 SIY 20100619          Switching to own implementation of character UI.
** 015 SIY 20100804          Changes in imports, some refactorings.
** 016 SIY 20110112          Changes related to new color handling scheme.
** 017 GES 20140627          Changed color attribute method names to match naming standards.
** 018 CA  20140711          Enhanced font support, for GUI clients.
** 019 HC  20140911          GUI coordinate support - data types refactoring.
** 020 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 021 SVL 20141102          Added some attributes.
*/

package com.goldencode.p2j.ui;

import java.io.*;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.format.*;

/**
 * Attribute container for the BrowseColumn widget.
 */
public class BrowseColumnConfig
extends WidgetConfig
{
   // start of unused/unimplemented fields

   /** DBNAME attribute variable */
   public String dbname = null;

   /** LABEL-DCOLOR attribute variable */
   public Color labelDColor = Color.NORMAL; // LABEL_DCOLOR;

   /** MODIFIED attribute variable */
   public boolean modified = false;

   /** AUTO-VALIDATE attribute variable */
   public boolean autoValidate = false;

   /** ROW attribute variable */
   public double row = 0;

   // end of unused fields
   
   // start of client-side only fields
   
   /** COLUMN attribute variable */
   public double column = 0;

   // end of client-side only fields

   /** COLUMN-DCOLOR attribute variable */
   public Color columnDColor = Color.NORMAL; // COLUMN_DCOLOR;

   /** COLUMN-PFCOLOR attribute variable */
   public Color columnPfColor = Color.INPUT; // COLUMN_PFCOLOR;

   /** COLUMN-BGCOLOR attribute variable */
   public Color columnBgColor = Color.INPUT;

   /** WIDTH-CHARS attribute variable */
   public double widthChars = -1;

   /** READ-ONLY attribute variable */
   public boolean readOnly = true;   //By default column is read-only

   /** AUTO-ZAP attribute. */
   public boolean autoZap = true;

   /** AUTO-RESIZE attribute variable */
   public boolean autoResize = true;

   /** AUTO-RETURN attribute variable */
   public boolean autoReturn = false;

   /** DATA-TYPE attribute variable */
   public String dataType = null;

   /** HELP attribute variable */
   public String help = null;

   /** LABEL attribute variable */
   public String label = null;

   /** COLUMN-LABEL attribute variable */
   public String clabel;

   /** Column display format. */
   public String format = null;

   /** Link to browse widget. */
   public int browseId = -1;

   /** Column ordinal in the browse definition */
   public int ordinal = -1;

   /** Propagated LABELS attribute */
   public boolean labels = true;

   /** Force label. */
   public boolean forceLabel = false;

   /**
    * Format width of base AT field.
    */
   public int formatWidth = -1;
   
   /** The label font for this browse column. */
   public int labelFont = FontManager.UNDEFINED_FONT;
   
   /** The column font for this browse column.*/
   public int columnFont = FontManager.UNDEFINED_FONT;

   /** LABEL-BGCOLOR attribute. */
   public Color labelBgColor = Color.NORMAL;

   /** SORT attribute. */
   public boolean sort = false;

   /** MAX-CHARS attribute. Applies only to combo-box browse columns. */
   public int maxChars = 255;
   
   /**
    * Default constructor (only used in deserialization).
    */
   public BrowseColumnConfig()
   {
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected BrowseColumnConfig(int id)
   {
      super(id);
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected BrowseColumnConfig(WidgetId id)
   {
      super(id);
   }

   /**
    * Synchronize the configuration with the widget state.
    */
   @Override
   public void syncWithWidget()
   {
      super.syncWithWidget();
      
      if (onClientSide())
      {
         // nothing for the client-side
         return;
      }

      // the LABELS attribute is inherited from the parent browse
      if (browseId != -1)
      {
         BrowseConfig bcfg = ConfigManager.getInstance().getActiveConfig(new WidgetId(browseId));
         labels = bcfg.labels;
      }
      
      // check if width-chars can be set
      double colWidth = widthChars;

      if (autoResize)
      {
         DisplayFormat colFormat = Browse.getDisplayFormat(this);
         String[] colLabel = Browse.getLabelVar(this);
         
         colWidth = colFormat.getScreenWidth();
         
         if (labels || forceLabel)
         {
            for (int i = 0; i < colLabel.length; ++i)
            {
               if (colLabel[i].length() > colWidth)
                  colWidth = colLabel[i].length();
            }
         }
      }

      this.widthChars = Coordinate.round(Math.max(formatWidth, colWidth));

      if (!labels && !forceLabel)
         label = "";
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      BrowseColumnConfig cfg = (BrowseColumnConfig) config;
      
      dbname = cfg.dbname;
      labelDColor = cfg.labelDColor;
      modified = cfg.modified;
      autoValidate = cfg.autoValidate;
      row = cfg.row;
      column = cfg.column;
      columnDColor = cfg.columnDColor;
      columnPfColor = cfg.columnPfColor;
      widthChars = cfg.widthChars;
      readOnly = cfg.readOnly;
      autoZap = cfg.autoZap;
      autoResize = cfg.autoResize;
      autoReturn = cfg.autoReturn;
      dataType = cfg.dataType;
      help = cfg.help;
      label = cfg.label;
      clabel = cfg.clabel;
      format = cfg.format;
      browseId = cfg.browseId;
      ordinal = cfg.ordinal;
      labels = cfg.labels;
      forceLabel = cfg.forceLabel;
      formatWidth = cfg.formatWidth;
      labelFont = cfg.labelFont;
      columnFont = cfg.columnFont;
      columnBgColor = cfg.columnBgColor;
      labelBgColor = cfg.labelBgColor;
      sort = cfg.sort;
      maxChars = cfg.maxChars;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      dbname             = (String) in.readObject();
      modified           = in.readBoolean();
      column             = in.readDouble();
      labelDColor        = (Color) in.readObject();
      columnDColor       = (Color) in.readObject();
      columnPfColor      = (Color) in.readObject();
      row                = in.readDouble();
      widthChars         = in.readDouble();
      browseId           = in.readInt();
      ordinal            = in.readInt();
      formatWidth        = in.readInt();
      readOnly           = in.readBoolean();
      autoValidate       = in.readBoolean();
      autoZap            = in.readBoolean();
      autoResize         = in.readBoolean();
      autoReturn         = in.readBoolean();
      labels             = in.readBoolean();
      forceLabel         = in.readBoolean();
      dataType           = (String) in.readObject();
      help               = (String) in.readObject();
      label              = (String) in.readObject();
      clabel             = (String) in.readObject();
      format             = (String) in.readObject();
      labelFont          = in.readInt();
      columnFont         = in.readInt();
      columnBgColor      = (Color) in.readObject();
      labelBgColor       = (Color) in.readObject();
      sort               = in.readBoolean();
      maxChars           = in.readInt();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeObject(dbname);
      out.writeBoolean(modified);
      out.writeDouble(column);
      out.writeObject(labelDColor);
      out.writeObject(columnDColor);
      out.writeObject(columnPfColor);
      out.writeDouble(row);
      out.writeDouble(widthChars);
      out.writeInt(browseId);
      out.writeInt(ordinal);
      out.writeInt(formatWidth);
      out.writeBoolean(readOnly);
      out.writeBoolean(autoValidate);
      out.writeBoolean(autoZap);
      out.writeBoolean(autoResize);
      out.writeBoolean(autoReturn);
      out.writeBoolean(labels);
      out.writeBoolean(forceLabel);
      out.writeObject(dataType);
      out.writeObject(help);
      out.writeObject(label);
      out.writeObject(clabel);
      out.writeObject(format);
      out.writeInt(labelFont);
      out.writeInt(columnFont);
      out.writeObject(columnBgColor);
      out.writeObject(labelBgColor);
      out.writeObject(sort);
      out.writeInt(maxChars);
   }
   
   /**
    * Initialize this config.
    */
   protected void init()
   {
      // this is always true; is forced at the widget (server and client-side)
      this.visible =  true; // direct access 
      this.hidden = false; // direct access
   }
}
