/*
** Module   : ComboBoxWidget.java
** Abstract : server side combo box widget implementation
**
** Copyright (c) 2005-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026   @23483 Created initial version.
** 002 SIY 20050209   @24433 Major refactoring.
** 003 SIY 20050209   @24494 Added missing setInnerLines method.
** 004 GES 20060803   @28368 Some cleanup and refactoring.
** 005 GES 20080320   @37583 Added TYPE attribute.
** 006 SIY 20111101          Updated imports.
** 007 CA  20130221          Refactored usage of TYPE attribute.
** 008 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 009 CA  20130927          Added LegacyResource annotation, to determine resource type.
** 010 CA  20131115          Changes related to properly instantiate dynamic resources.
** 011 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 012 SVL 20141102          Added MAX-CHARS and AUTO-ZAP attributes.
*/ 

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Server side combo box widget implementation.
 */
@LegacyResource(resource = LegacyResource.COMBO_BOX)
public class ComboBoxWidget
extends ControlSetEntity<ComboBoxConfig>
implements ComboBoxElement,
           AutoZapElement
{
   /**
    * Default constructor.
    */
   public ComboBoxWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public ComboBoxWidget(boolean dynamic)
   {
      super(dynamic, new ComboBoxConfig());
   }
   
   /**
    * Set DROP-DOWN-LIST mode for the widget. Has no effect in character mode
    * interface.
    * 
    * @param   b
    *          <code>true</code> enables mode, <code>false</code> is
    *          ignored.
    */
   public void setModeDropDownList(boolean b)
   {
      // do nothing
   }

   /**
    * Set DROP-DOWN mode for the widget. Has no effect in character mode
    * interface.
    * 
    * @param   b
    *          <code>true</code> enables mode, <code>false</code> is
    *          ignored.
    */
   public void setModeDropDown(boolean b)
   {
      // do nothing
   }
   
   /**
    * Set SIMPLE mode for the widget. Has no effect in character mode
    * interface.
    * 
    * @param   b
    *          <code>true</code> enables mode, <code>false</code> is
    *          ignored.
    */
   public void setModeSimple(boolean b)
   {
      // do nothing
   }

   /**
    * Set number of inner lines in the widget drop-down window.
    * 
    * @param   lines
    */
   public void setInnerLines(int lines)
   {
      config.innerLines = lines;
      pushScreenDefinition();
   }

   /**
    * Sets the DISABLE-AUTO-ZAP option.
    * 
    * @param    disableAutoZap
    *           <code>true</code> if the option is set to ON.
    */
   public void setDisableAutoZap(boolean disableAutoZap)
   {
      config.autoZap = !disableAutoZap;
      pushScreenDefinition();
   }

   /**
    * Set AUTO-ZAP attribute.
    * 
    * @param    value
    *           New value for the attribute.
    */
   public void setAutoZap(boolean value)
   {
      config.autoZap = value;
      pushScreenDefinition();
   }

   /**
    * Set value of the MAX-CHARS attribute.
    *
    * @param    maxChars
    *           Value of the attribute.
    */
   public void setMaxChars(double maxChars)
   {
      setMaxChars(new decimal(maxChars));
   }

   /**
    * Set value of the MAX-CHARS attribute.
    *
    * @param    maxChars
    *           Value of the attribute.
    */
   public void setMaxChars(NumberType maxChars)
   {
      if (maxChars.isUnknown())
         return;

      config.maxChars = maxChars.intValue();
      pushScreenDefinition();
   }

   /**
    * Get value of the MAX-CHARS attribute.
    *
    * @return   value of the attribute.
    */
   public integer getMaxChars()
   {
      return new integer(config.maxChars);
   }

   /**
    * Set value of the AUTO-ZAP attribute.
    *
    * @param    autoZap
    *           Value of the attribute.
    */
   public void setAutoZap(logical autoZap)
   {
      if (autoZap.isUnknown())
         return;

      setAutoZap(autoZap.booleanValue());
   }

   /**
    * Get value of the AUTO-ZAP attribute.
    *
    * @return   value of the attribute.
    */
   public logical isAutoZap()
   {
      return new logical(config.autoZap);
   }
}
